import sys

##Map the test cases' error names to the actual errors
#ErrorNames = {'clamped' : Clamped,
#              'conversion_syntax' : InvalidOperation,
#              'division_by_zero' : DivisionByZero,
#              'division_impossible' : InvalidOperation,
#              'division_undefined' : InvalidOperation,
#              'inexact' : Inexact,
#              'invalid_context' : InvalidOperation,
#              'invalid_operation' : InvalidOperation,
#              'overflow' : Overflow,
#              'rounded' : Rounded,
#              'subnormal' : Subnormal,
#              'underflow' : Underflow}
#
# The following functions return True/False rather than a Decimal instance

LOGICAL_FUNCTIONS = (
    'is_canonical',
    'is_finite',
    'is_infinite',
    'is_nan',
    'is_normal',
    'is_qnan',
    'is_signed',
    'is_snan',
    'is_subnormal',
    'is_zero',
    'same_quantum',
    )


changeFunct = {
    'precision'   : 'context.prec = %s',
    'rounding'    : 'context.rounding = %s',
    'maxexponent' : 'context.Emax = %s',
    'minexponent' : 'context.Emin = %s',
    'clamp'       : 'context._clamp = %s',
    'extended'    : None,
    'version'     : None,
    'dectest'     : None,
}

changeRounding = {
    'ceiling'   : 'ROUND_CEILING',
    'down'      : 'ROUND_DOWN',
    'floor'     : 'ROUND_FLOOR',
    'half_down' : 'ROUND_HALF_DOWN',
    'half_even' : 'ROUND_HALF_EVEN',
    'half_up'   : 'ROUND_HALF_UP',
    'up'        : 'ROUND_UP',
    '05up'      : 'ROUND_05UP',
}

nameAdapter = {
    'abs':'__abs__',
    'add':'__add__',
    'and':'logical_and',
    'apply':None,
    'canonical':'canonical',
    'class':'number_class',
    'compare':'compare',
    'comparesig':'compare_signal',
    'comparetotal':'compare_total',
    'comparetotmag':'compare_total_mag',
    'copy':None,
    'copyabs':'copy_abs',
    'copynegate':'copy_negate',
    'copysign':'copy_sign',
    'divide':'__div__',
    'divideint':'__floordiv__',
    'exp':'exp',
    'fma':'fma',
    'invert':'logical_invert',
    'iscanonical':'is_canonical',
    'isfinite':'is_finite',
    'isinfinite':'is_infinite',
    'isnan':'is_nan',
    'isnormal':'is_normal',
    'isqnan':'is_qnan',
    'issigned':'is_signed',
    'issnan':'is_snan',
    'issubnormal':'is_subnormal',
    'iszero':'is_zero',
    'ln':'ln',
    'log10':'log10',
    'logb':'logb',
    'max':'max',
    'maxmag':'max_mag',
    'min':'min',
    'minmag':'min_mag',
    'minus':'__neg__',
    'multiply':'__mul__',
    'nextminus':'next_minus',
    'nextplus':'next_plus',
    'nexttoward':'next_toward',
    'or':'logical_or',
    'plus':'__pos__',
    'power':'__pow__',
    'quantize':'quantize',
    'reduce':'normalize',
    'remainder':'__mod__',
    'remaindernear':'remainder_near',
    'rescale':None,
    'rotate':'rotate',
    'samequantum':'same_quantum',
    'scaleb':'scaleb',
    'shift':'shift',
    'squareroot':'sqrt',
    'subtract':'__sub__',
    'toeng':'to_eng_string',
    'tointegral':'to_integral_value',
    'tointegralx':'to_integral_exact',
    'tosci':'__str__',
    'xor':'logical_xor',
}

def fixQuotes(val):
    val = val.replace("''", 'SingleQuote').replace('""', 'DoubleQuote')
    val = val.replace("'", '').replace('"', '')
    val = val.replace('SingleQuote', "'").replace('DoubleQuote', '"')
    return val

def eval_line(s, withop):
    if s.find(' -> ') >= 0 and s[:2] != '--' and not s.startswith('  --'):
        s = (s.split('->')[0] + '->' +
             s.split('->')[1].split('--')[0]).strip()
    else:
        s = s.split('--')[0].strip()

    if not s:
        return

    if ':' in s:
        funct, value = map(lambda x: x.strip().lower(), s.split(':'))
#        print funct, value
        if funct == 'rounding':
            value = changeRounding[value]

        realfunct = changeFunct[funct]
        if not realfunct:
            return

        return realfunct % value

#    print s
    sides = s.split('->')
    tmp = sides[0].strip().split()
    funct = tmp[1].lower()
    valstemp = tmp[2:]

    # if we have exceptions, we skip this test
    # we could catch the exception, but timing would be distorted
    tmp = sides[1].strip().split()
    exceptions = tmp[1:]
    if exceptions:
        return

#    print funct, valstemp
    funct = nameAdapter[funct]

    if not funct:
        return

    valstemp = map(fixQuotes, valstemp)
    for val in valstemp:
        if "#" in val:
            return

    if not withop:
        orden = ";".join(["Decimal('%s')"]*len(valstemp))
        orden = orden % tuple(valstemp)
        return orden

    if len(valstemp) == 1:
        orden = "Decimal('%s').%s()" % (valstemp[0], funct)
    elif len(valstemp) == 2:
        orden = "Decimal('%s').%s(Decimal('%s'))" % (valstemp[0], funct, valstemp[1])
    elif len(valstemp) == 3:
        orden = "Decimal('%s').%s(Decimal('%s'), Decimal('%s'))" % (valstemp[0], funct, valstemp[1], valstemp[2])
    else:
        raise ValueError("valstemp largo mal en funct %r: %r" % (funct, valstemp))

    return orden

    funct = getattr(self.context, fname)
    vals = []
    conglomerate = ''
    quote = 0

    for i, val in enumerate(valstemp):
        if val.count("'") % 2 == 1:
            quote = 1 - quote
        if quote:
            conglomerate = conglomerate + ' ' + val
            continue
        else:
            val = conglomerate + val
            conglomerate = ''
        v = FixQuotes(val)
        v = Decimal(v, self.context)
        vals.append(v)

    try:
        result = str(funct(*vals))
        if fname in LOGICAL_FUNCTIONS:
            result = str(int(eval(result))) # 'True', 'False' -> '1', '0'
    except: #Catch any error long enough to state the test case.
        print "ERROR:", s
        raise

    return




def eval_file(write, testFile, withop):
    if withop:
        print "Eval:", testFile
    write("context = Context()")
    write("setcontext(context)")
    for line in open(testFile):
        line = line.replace('\r\n', '').replace('\n', '')
        res = eval_line(line, withop)
        if res is not None:
            write(res)
    return


class Formatter(object):
    def __init__(self, arch):
        self.arch = arch
        self.cantops = 0

    def write(self, msg):
        self.cantops += 1
        self.arch.write(msg+"\n")

    def separ(self, msg):
        self.arch.write("\n\n#--- " + msg + "\n")

def main(nomout, nomtests):
    arch = open(nomout, "w")
    arch.write("import time\n")
    arch.write("from decimal import *\n\n")
#    arch.write("DefaultContext.prec = 9\n")
#    arch.write("DefaultContext.rounding = ROUND_HALF_EVEN\n")
#    arch.write("DefaultContext.traps = dict.fromkeys(Signals, 0)\n")
    arch.write("tini = time.clock()\n")

    f = Formatter(arch)
    
    for test in nomtests:
        f.separ("No operation -- " + str(test))
        eval_file(f.write, test, withop=False)

    arch.write("tinterm = time.clock()\n")

    f.cantops = 0
    for test in nomtests:
        f.separ(str(test))
        eval_file(f.write, test, withop=True)

    arch.write("\n\noverhead = tinterm - tini\n")
    arch.write("ttotal = (time.clock() - tinterm - overhead)*1000\n")
    arch.write("\nprint 'Operations: %5d'\n" % f.cantops)
    arch.write("print 'Time:        %7.2f mseg' % ttotal\n")
    arch.write("print 'Per Op       %%7.2f useg' %% (1000*ttotal/%d)\n" % f.cantops)
    arch.close()
    

if __name__ == "__main__":
    if len(sys.argv) < 3:
        print "Use:  genDecimalBenchTest.py outfile testfile1 [testfile2 [...]]"
        sys.exit()

    main(sys.argv[1], sys.argv[2:])
